from zope.interface import implements
from zope.component import adapts
from Products.ATContentTypes.interface.image import IImageContent
from Products.ImageEditor.interfaces import IImageEditorAdapter
from PIL import Image, ImageFilter, ImageEnhance
from cStringIO import StringIO

class ImageEditorAdapter(object):
    """context.getField('image').get(context) -> Field Image
    Field Image.data -> OFS Image
    OFSImage.data    -> str
    
    stack = [str, ...]
    """
    implements(IImageEditorAdapter)
    adapts(IImageContent)
    
    def __init__(self, context):
        self.context = context

        if not hasattr(context, 'stack_pos'):
            self.pos = 0
        if not hasattr(context, 'unredostack'):
            self.stack = [self.get_image_data()]

    def get_image_data(self):
        data = self.context.getField('image').get(self.context).data
        if type(data) == str:
            return data
        else:
            return data.data

    #UNDO REDO STUFF
    def get_pos(self):
        return self.context.stack_pos
    def set_pos(self, value):
        self.context.stack_pos = value
    pos = property(get_pos, set_pos)

    def get_stack(self):
        return self.context.unredostack
    def set_stack(self, value):
        self.context.unredostack = value
    stack = property(get_stack, set_stack)

    def undo(self):
        if self.can_undo():
            self.pos = self.pos - 1

    def can_undo(self):
        return self.pos > 0

    def redo(self):
        if self.can_redo():
            self.pos = self.pos + 1

    def can_redo(self):
        return self.pos + 1 < len(self.stack)

    def clear_edits(self, bottom=None):
        if hasattr(self.context, 'stack_pos'):
            delattr(self.context, 'stack_pos')

        if hasattr(self.context, 'unredostack'):
            delattr(self.context, 'unredostack')

        self.pos = 0

        if bottom is None:
            bottom = self.get_image_data()

        self.stack = [bottom]


    def do(self, value):
        if self.can_redo():
            for item in self.stack[(self.pos+1):len(self.stack)]:
                self.stack.remove(item)

        self.stack.append(value)
        self.pos = self.pos + 1

    def save_edit(self):
        image_data = self.get_current_image_data()
        
        field = self.context.getField('image')
        mimetype = field.getContentType(self.context)
        filename = field.getFilename(self.context)
        
        # because AT tries to get mimetype and filename from a file like
        # object by attribute access I'm passing a string along
        field.set(
            self.context,
            image_data, 
            mimetype=mimetype,
            filename=filename, 
            refresh_exif=False
        )
        
        #should I clear???  Maybe you should still be able to undo...              
        self.clear_edits(image_data)

    def get_current_image(self):
        return Image.open(StringIO(self.get_current_image_data()))

    def get_current_image_data(self):
        return self.stack[self.pos]

    def get_current_image_info(self):
        data = self.get_current_image_data()
        bsize = len(data)
        bsize = bsize/1024

        size_descriptor = 'kb'
        if bsize > 1024:
            bsize = bsize/1024.0
            size_descriptor = 'mb'

        size = self.get_current_image().size

        width = size[0]
        height = size[1]

        return {
            'size': bsize,
            'width': width,
            'height': height,
            'sizeformatted': "Size: %.2f%s" % (bsize, size_descriptor)
        }


    def set_image(self, image, format="JPEG", quality=None):
        image_data = StringIO()
        
        if quality:
            image.save(image_data, format, quality=quality)
        else:
            image.save(image_data, format)
        
        self.do(image_data.getvalue())





import random

def normalize_name(name):
    return name.lower().replace(' ', '-').replace(':', "-")

def generate_random_url(context):
    return context.absolute_url() + "/currenteditedimage.jpg?" + str(random.randint(0, 1000000))
    
def get_image_information(editor):
    image_info = editor.get_current_image_info()
    return {
        'url' : generate_random_url(editor.context),
        'can_undo' : editor.can_undo(),
        'can_redo' : editor.can_redo(),
        'can_save' : editor.pos > 0,
        'size' : image_info['sizeformatted'],
        'width' : image_info['width'],
        'height' : image_info['height']
    }
    
def json(d):
    
    def convert_type(value):
        if type(value) == bool:
            return str(value).lower()
        elif type(value) == str:
            return "'%s'" % value
        elif type(value) == dict:
            return json(value)
        elif type(value) == list or type(value) == tuple:
            return "[%s]" % ','.join([convert_type(v) for v in value])
        else:
            return value
        
    return "{%s}" % (', '.join(["%s : %s" % (name, convert_type(value)) for name, value in d.items()]))
    
    


from zope.i18nmessageid import MessageFactory
imageeditor_message_factory = MessageFactory('Products.ImageEditor')

import os

cwd = os.sep.join(__file__.split(os.sep)[:-1])
dependencies = [d.strip('\n').strip() for d in open(os.path.join(cwd, 'dependencies.txt')).readlines() ]


from zope.interface import Interface, Attribute

class IImageEditorAdapter(Interface):

    pos = Attribute("""Position of the stack""")
    stack = Attribute("""Actual stack...""")

    def do(self, value):
        """performing the action.
        value: str image data
        """

    def can_undo(self):
        """
        Returns boolean if they can undo
        """

    def undo():
        """
        undo last edit
        """

    def can_redo(self):
        """
        Returns boolean if they can redo
        """

    def redo():
        """
        redo edit
        """
        
    def clear_edits():
        """
        clear all edits from undo/redo
        """

    def save_edit():
        """
        This method gets the current image in the unredo stack and saves it to the
        object.  It then saves the history of it and gets the version_message so
        the history has better info...
        """
       
    def set_image(image, format="JPEG", quality=None):
        """ Set new image on stack
        image: PIL.image
        format: str represent the format
        quality: float for quality if supported by format
        """
       
    def get_current_image():
        """ return the currently edited image
          -> PIL.Image
        """
        
    def get_current_image_data():
       """ get currently edited image data
         -> str
       """
       
    def get_current_image_info():
        """ Return a dict of info about the image
         -> {'size': , 
             'width':,
             'height': ,
             'sizeformatted': "Size: %s%s"
            }
        
        """
       
class IImageEditorUtility(Interface):
    
    def should_include(context):
        """
        will check if css and js files should be included
        """
        
class IImageEditorLayer(Interface):
    """
    marker interface for image editor layer
    """
    
class IImageEditorContext(Interface):
    """
    Just a context for image editor actions so they can be traversed.
    """
    
class IImageEditorActionContext(Interface):
    """
    The actually traversable actions that allow ajax calls to be performed.
    """


from zope.interface import Interface, Attribute

class IImageEditorAction(Interface):
    """
    The base action for all image editor actions
    """
    name = Attribute("""The name of the action.""")
    description = Attribute("""A description of what it does.""")
    skip_apply = Attribute("""Whether the button should skip the apply button and immediately perform.""")
    options = Attribute("""The FormFields of options that the action has.""")
    icon = Attribute("""The icon for the button to use.""")
    
    def action_parameters():
        """
        javascript that will gather the parameters to be sent
        to request.  Return false and do not implement to just use
        standard options.  This is mostly just for complex actions.
        The javascript here should be a method that returns a JSON
        object of parameters.
        """
        
    def on_setup():
        """
        extra javascript that will be added for any extra setup needed
        for button actions.
        """
        
    def __call__():
        """
        This is the heavy lifter and will perform the actual action.
        """


from zope.app.schema.vocabulary import IVocabularyFactory
from zope.interface import implements
from Products.Archetypes.mimetype_utils import getAllowableContentTypes
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from Products.ATContentTypes.interface.image import IImageContent
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.ArchetypeTool import _types as registered_types

class ImageContentTypeVocabulary(object):
    implements(IVocabularyFactory)

    def __call__(self, context):    
        type_values = registered_types.values()
        impl_image = lambda x : IImageContent.implementedBy(x['klass'])
        image_types = [v['klass'].portal_type for v in type_values if impl_image(v)]
        items = [SimpleTerm(i, i , i) for i in image_types]
        return SimpleVocabulary(items)

ImageContentTypeVocabularyFactory = ImageContentTypeVocabulary()


from setuptools import setup, find_packages
import os

dependencies = [
    'setuptools'
]

#now get plone dependencies
dep_file = open(os.path.join('Products', 'ImageEditor', 'dependencies.txt'))
dependencies.extend([ d.strip('\n').strip() for d in dep_file.readlines() ])

setup(name='Products.ImageEditor',
      version='1.2rc1',
      description="A product that adds an editor tab to ATImage and News Item so you can rotate, flip, blur, compress, contrast, brightness, sharpen, add drop shadows, crop, resize an image.",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='atimage plone image rotate flip blur compress contrast brightness sharpen drop shadows crop resize',
      author='Nathan Van Gheem',
      author_email='vangheem@gmail.com',
      url='http://www.plone.org/products/products-imageeditor',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=dependencies,
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



import unittest
import sys
from cStringIO import StringIO

from PIL import Image
from PIL import ImageChops
from PIL import ImageEnhance

from Products.ImageEditor.adapters.imageeditor import ImageEditorAdapter

from zope.testing import doctestunit
from zope.component import testing
from Testing import ZopeTestCase as ztc

from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import PloneSite
from Products.PloneTestCase.layer import onsetup
from Products.CMFCore.utils import getToolByName

from zope.app import zapi
from zope.configuration import xmlconfig

BASE_DIR = ""

for path in sys.path:
    if 'Products.ImageEditor' in path:
        BASE_DIR = path

import Products.ImageEditor, collective.js.jqueryui, collective.js.jquery

@onsetup
def setUp():
    fiveconfigure.debug_mode = True
    zcml.load_config('configure.zcml', Products.ImageEditor)
    zcml.load_config('configure.zcml', collective.js.jquery)
    zcml.load_config('configure.zcml', collective.js.jqueryui)
    fiveconfigure.debug_mode = False
    ztc.installProduct('ImageEditor')
    ztc.installPackage('collective.js.jquery')
    ztc.installPackage('collective.js.jqueryui')

setUp()

ptc.setupPloneSite(extension_profiles=('Products.ImageEditor:default',))

class ImageEditorTestCase(ptc.PloneTestCase):
    """
    """
    def afterSetUp(self):
        self.setRoles(('Manager',))
        
    def getImageContentType(self):
        try:
            id = self.portal.invokeFactory(type_name="Image", id="testimage")
        except Exception:
            id = 'testimage'
        image = self.portal[id]
        image.setTitle('test')
        #(Pdb) p context.getField('image').get(context)
        #<Image at /plone/bug-mantis-01.png/image>
        #(Pdb) p context.getField('image').get(context).data
        #<OFS.Image.Pdata object at 0xb277db6c>
        #(Pdb) p context.getField('image').get(context).data.data
        #'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x04N\x00\x00...
        #...
        #
        #In testcase:
        #(Pdb) image.getField('image').get(image)                                                                         
        #<Image at /plone/testimage/image>                                                                                
        #(Pdb) image.getField('image').get(image).data                                                                    
        #'\xff\xd8\xff\xe0\x00\x10JFIF\x00\x01\...
        im = self.getOriginal()
        imageData = StringIO()
        im.save(imageData, im.format)
        image.setImage(imageData)
        return image
    
    def uninstall(self):
        setup_tool = getToolByName(self.portal, 'portal_setup')
        setup_tool.runAllImportStepsFromProfile('profile-Products.ImageEditor:uninstall')
    
    def getEditor(self, context):
        return ImageEditorAdapter(context)
    
    def getImageEditorAdapter(self):
        image = self.getImageContentType()
        return ImageEditorAdapter(image)
               
    def getOriginal(self):
        return self.getImage('original.jpg')
        
    def getImagePath(self, name):
        return '%s/Products/ImageEditor/tests/testimages/%s' % (BASE_DIR, name)
        
    def getImage(self, name):
        return Image.open(self.getImagePath(name))



from Products.ImageEditor.utils import normalize_name
from zope.interface import Interface
from zope.configuration.fields import GlobalObject, GlobalInterface
from zope.configuration.fields import Tokens, Path, Bool, PythonIdentifier
from zope.configuration.fields import MessageID
from zope.schema import Text, TextLine, Id
from zope.security.zcml import Permission
from zope.app.component.back35 import LayerField
from zope.app.publisher.browser.fields import MenuField

from Products.ImageEditor.interfaces.actions import IImageEditorAction

class IEditorAction(Interface):
    """
    Fields to create an image editor action
    """

    class_ = GlobalObject(
        title=u"Class",
        description=u"The actual class that will be called when this action is selected",
        required=True
    )
    
    before = TextLine(
        title=u"Before",
        description=u"Insert this action before another action.",
        required=False
    )
    
    after = TextLine(
        title=u"After",
        description=u"Insert this action after another action.",
        required=False
    )
    

IMAGE_EDITOR_ACTIONS = {}
ACTION_ORDER = []

def get_actions():
    for name in ACTION_ORDER:
        yield name, IMAGE_EDITOR_ACTIONS[name]
    
def get_action_class(name):
    if IMAGE_EDITOR_ACTIONS.has_key(name):
        return IMAGE_EDITOR_ACTIONS[name].class_
    else:
        raise Exception("Can't find action %s" % name)
        
def get_action_names():
    for name in ACTION_ORDER:
        yield name
        
class ImageEditAction:

    def __init__(self, class_, before=None, after=None):
        self.name = normalize_name(class_.name)
        self.class_ = class_
        self.before = before
        self.after = after


def add_action(_context, class_, before=None, after=None):
    
    action = ImageEditAction(class_, before, after)
    
    IMAGE_EDITOR_ACTIONS[action.name] = action
    
    if not IImageEditorAction.implementedBy(action.class_):
        raise Exception("Class must implement IImageEditorAction")
    
    if action.before is not None:
        if action.before == "*":
            ACTION_ORDER.insert(0, action.name)
        else:
            for index in range(0, len(ACTION_ORDER)):
                if action.before == ACTION_ORDER[index]:
                    ACTION_ORDER.insert(index, action.name)
                    break
                    
    elif action.after is not None:
        if action.after == "*":
            ACTION_ORDER.insert(len(ACTION_ORDER), action.name)
        else:
            for index in range(0, len(ACTION_ORDER)):
                if action.after == ACTION_ORDER[index]:
                    ACTION_ORDER.insert(index+1, action.name)
                    break
                    
    else:
        ACTION_ORDER.append(action.name)
    